library fis_theme;

import 'package:fis_common/helpers/color.dart';
import 'package:fis_common/logger/logger.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';

import 'manifest.dart';
import 'text_size_scheme.dart';
import 'theme_data.dart';
import 'theme_loader.dart';
import 'resource.dart';
import 'theme_color_scheme.dart';

export 'resource.dart';
export 'resource_image_provider.dart';
export 'manifest.dart';
export 'font_loader.dart';
export 'locale_setting.dart';
export 'text_size_scheme.dart';
export 'theme_data.dart';
export 'theme_loader.dart';
export 'font_loader.dart';

abstract class FTheme {
  FTheme({required String defaultName, required ThemeMode themeMode}) {
    _defaultName = defaultName;
    _themeMode = themeMode;
  }

  /* 静态单例 */
  static bool _hasInit = false;
  static FTheme? _instance;

  /// 获取单例
  static FTheme get ins {
    if (!_hasInit) {
      throw FlutterError("FTheme has not been init.");
    }
    return _instance!;
  }

  /// 单例初始化
  static Future<void> init(FTheme instance, [String name = "lightness"]) async {
    if (!_hasInit) {
      _instance = instance;
      _hasInit = true;
      await instance.switchTheme(name);
    }
  }
  /* 静态单例 end */

  /// 当前语言是否中文
  bool get isCurrentChinese;

  /// 当前主题模式
  Brightness get currentBrightness;

  FThemeManifestInfo? _manifestInfo;

  /// 主题清单
  FThemeManifestInfo get manifestInfo => _manifestInfo!;

  late String _defaultName;

  late ThemeMode _themeMode;

  /// 主题明暗模式
  ThemeMode get themeMode => _themeMode;

  /// 默认主题名
  // @protected
  String get defaultName => _defaultName;

  late FThemeData _data;

  /// 主题数据
  FThemeData get data => _data;

  /// 是否深色模式
  bool get isDarkMode {
    if (themeMode == ThemeMode.system) {
      return currentBrightness == Brightness.dark;
    } else {
      return themeMode == ThemeMode.dark;
    }
  }

  /// 颜色组合
  FThemeColorScheme get colorScheme {
    return isDarkMode ? _data.darkColorScheme : _data.colorScheme;
  }

  /// 文字尺寸组合
  FThemeTextSizeScheme get textSizeScheme => _data.textSizeScheme;

  late ThemeData _innerThemeData;
  late ThemeData _innerDarkThemeData;

  /// 内置主题数据
  ThemeData get innerThemeData => getInnerThemeData(currentBrightness);

  /// 应用主题数据
  @protected
  void applyThemeData();

  /// 切换主题模式
  Future<void> switchThemeMode(ThemeMode mode) async {
    _themeMode = mode;
  }

  /// 根据明暗模式获取内置主题数据
  // @protected
  ThemeData getInnerThemeData(Brightness brightness) =>
      brightness == Brightness.dark ? _innerDarkThemeData : _innerThemeData;

  /// 重新加载
  Future<void> reload() async {
    await buildThemeData(this.data.name);
    applyThemeData();
  }

  /// 切换主题
  Future<bool> switchTheme([String? name]) async {
    bool loaded = true;
    if (name != null) {
      loaded = await loadManifestInfo(name);
    } else {
      name = this.data.name;
    }
    if (loaded) {
      await buildThemeData(name);
      applyThemeData();
      return true;
    }
    return false;
  }

  /// 加载主题清单
  @protected
  Future<bool> loadManifestInfo([String? name]) async {
    try {
      FThemeManifestInfo? manifestInfo =
          await getThemeLoader(name ?? defaultName).load();
      if (manifestInfo != null) {
        _manifestInfo = manifestInfo;
        fRootBundle.init(manifestInfo);
        return true;
      }
    } catch (e) {
      logger.e("加载主题清单失败", e);
    }
    return false;
  }

  @protected
  Future<void> buildThemeData(String name) async {
    _data = FThemeData(name, this.manifestInfo);
    await buildInnerThemeData();
  }

  @protected
  Future<void> buildInnerThemeData() async {
    final localeSetting = this.isCurrentChinese
        ? this.data.localeSettings.chinese
        : this.data.localeSettings.english;
    final textSizeScheme = this.data.textSizeScheme;

    _innerThemeData = _createThemeData(
      Brightness.light,
      this.data.colorScheme,
      textSizeScheme,
      localeSetting.fontFamily,
    );
    _innerDarkThemeData = this.data.hasDarkColorScheme
        ? _createThemeData(
            Brightness.dark,
            this.data.darkColorScheme,
            textSizeScheme,
            localeSetting.fontFamily,
          )
        : _innerThemeData;
  }

  ThemeData _createThemeData(
      Brightness brightness,
      FThemeColorScheme colorScheme,
      FThemeTextSizeScheme textSizeScheme,
      String fontFamily) {
    var textTheme = TextTheme(
      headline3: TextStyle(fontSize: textSizeScheme.headlineLargeSize), // 大标题
      headline4: TextStyle(fontSize: textSizeScheme.headlineSize), // 标题
      headline5: TextStyle(fontSize: textSizeScheme.headlineSmallSize), // 小标题
      bodyText2: TextStyle(fontSize: textSizeScheme.bodySize), // 正文
      button: TextStyle(fontSize: textSizeScheme.buttonSize), // 按钮文字
    );
    textTheme.apply(fontFamily: fontFamily);

    return ThemeData(
      brightness: brightness,
      fontFamily: fontFamily,
      primarySwatch: FColorHelper.createMaterialColor(colorScheme.primary),
      accentColor: colorScheme.foreground, //前景色
      backgroundColor: colorScheme.background, //背景色
      buttonColor: colorScheme.primary, //按钮色
      canvasColor: colorScheme.background, //基础色
      dialogBackgroundColor: colorScheme.background, //Dialog背景色
      disabledColor: colorScheme.line, // 禁用色
      errorColor: colorScheme.additional, //错误色
      hintColor: colorScheme.textTips, //提示或占位文字色
      selectedRowColor: colorScheme.secondary, //选中色
      splashColor: colorScheme.secondary, //InkWall色
      textTheme: textTheme,
    );
  }
}
