import 'dart:convert';
import 'dart:io';

import 'package:archive/archive.dart';
import 'package:dio/adapter.dart';
import 'package:dio/dio.dart';
import 'package:fis_common/env/env.dart';
import 'package:fis_common/helpers/http.dart';
import 'package:fis_common/logger/logger.dart';
import 'package:fis_jsonrpc/exception.dart';
import 'package:fis_jsonrpc/request.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';
import 'package:path_provider/path_provider.dart';

import 'theme.dart';
import 'consts.dart';
import 'resource.dart';
import 'manifest.dart';

abstract class FThemeLoader {
  /// 主题名称
  final String name;
  FThemeLoader(this.name);

  /// 加载主题清单
  Future<FThemeManifestInfo?> load() async {
    bool isDefault = FTheme.ins.defaultName == this.name;
    if (!isDefault) {
      bool pkgLoaded = await _loadPackage();
      if (!pkgLoaded) {
        logger.w("加载主题包失败");
        return null;
      }
    }
    String? manifestJson =
        isDefault ? await _getDefaultManifestJson() : await _getManifestJson();
    if (manifestJson == null) {
      logger.w("主题清单json为空");
      return null;
    }

    try {
      Map<String, dynamic> json = jsonDecode(manifestJson);
      FThemeManifestInfo manifestInfo = FThemeManifestInfo.fromJson(json);
      return manifestInfo;
    } catch (e) {
      logger.e("解析主题清单Json失败", e);
    }
    return null;
  }

  /// 加载主题包
  Future<bool> _loadPackage();

  /// 获取清单Json字符串
  Future<String?> _getManifestJson();

  /// 获取默认主题清单Json字符串
  Future<String?> _getDefaultManifestJson() async {
    try {
      String assetPath =
          "packages/${FisUIConsts.themePackageName}/assets/${FisUIConsts.manifestFileName}";
      String jsonString = await rootBundle.loadString(assetPath, cache: false);
      return jsonString;
    } catch (e) {
      logger.e("读取默认主题清单Json失败", e);
    }
    return null;
  }
}

/// 原生平台(iOS/Android)主题加载器
class FThemeLoaderNative extends FThemeLoader {
  FThemeLoaderNative(String name) : super(name);

  Directory? _storageDirectory;

  @override
  Future<bool> _loadPackage() async {
    try {
      final String storagePath = await _getStoragePath();
      final String pkgFileName = "$storagePath/themes/${this.name}.zip";
      if (!await File(pkgFileName).exists()) {
        final String url = _getPackageUrl();
        final Dio dio = Dio(BaseOptions(connectTimeout: 10 * 1000));
        (dio.httpClientAdapter as DefaultHttpClientAdapter).onHttpClientCreate =
            (client) {
          // handle for cors
          client.badCertificateCallback =
              (X509Certificate cert, String host, int port) => true;
        };
        var response = await dio.download(url, pkgFileName);
        if (response.statusCode == null ||
            response.statusCode != HttpStatus.ok) {
          logger.w("下载主题包失败, Http code - ${response.statusCode}");
          return false;
        }
      }
      final zipBytes = await File(pkgFileName).readAsBytes();
      final archive = ZipDecoder().decodeBytes(zipBytes);
      for (final file in archive) {
        final filePath = "$storagePath/themes/${this.name}/${file.name}";
        if (file.isFile) {
          final data = file.content as List<int>;
          File fileInfo = File(filePath);
          await fileInfo.create(recursive: true);
          await fileInfo.writeAsBytes(data);
        } else {
          await Directory(filePath).create(recursive: true);
        }
      }
      return true;
    } catch (e) {
      print(e);
      logger.e("加载主题包异常", e);
    }
    return false;
  }

  @override
  Future<String?> _getManifestJson() async {
    try {
      final String storagePath = await _getStoragePath();
      String filePath =
          "$storagePath/themes/${this.name}/${FisUIConsts.manifestFileName}";
      String json = await File(filePath).readAsString();
      return json;
    } catch (e) {
      logger.e("读取主题清单Json异常", e);
    }
    return null;
  }

  /// 获取主题文件存储路径
  Future<String> _getStoragePath() async {
    if (_storageDirectory == null) {
      _storageDirectory = await getApplicationDocumentsDirectory();
    }
    return _storageDirectory!.path;
  }

  /// 获取主题包链接地址
  String _getPackageUrl() => "${fRootBundle.resourcePath}/${this.name}.zip";
}

/// Web主题加载器
class FThemeLoaderWeb extends FThemeLoader {
  FThemeLoaderWeb(String name) : super(name);

  @override
  Future<String?> _getManifestJson() async {
    try {
      String url =
          "${getUrlRoot()}/${this.name}/${FisUIConsts.manifestFileName}";
      String? json = await FHttpHelper.downloadString(url);
      return json;
    } catch (e) {
      logger.e("读取主题清单Json异常", e);
    }
    return null;
  }

  @override
  Future<bool> _loadPackage() async => true;

  /// 获取链接头
  @protected
  String getUrlRoot() => fRootBundle.resourcePath;
}

/// 套壳Web(Win/Mac)主题加载器
class FThemeLoaderShellWeb extends FThemeLoaderWeb {
  FThemeLoaderShellWeb(String name) : super(name);

  @override
  Future<bool> _loadPackage() async {
    try {
      final url = "http://platform.fis.plus/IPlatformService";
      final rpcRequest = JsonRpcRequest("LoadTheme", name);
      final String package = jsonEncode(rpcRequest.toJson());
      final dio = Dio();
      dio.options.sendTimeout = 30 * 1000;
      dio.options.contentType = Headers.jsonContentType;
      dio.options.responseType = ResponseType.json;
      final response = await dio.post(url, data: package);
      if (response.statusCode == HttpStatus.ok && response.data != null) {
        var resp = response.data;
        if (resp.containsKey('error')) {
          throw JsonRpcServerError.fromJson(resp['error']);
        }
        return resp['result'];
      }
    } catch (e) {
      logger.e("加载主题包异常", e);
    }
    return false;
  }

  @override
  String getUrlRoot() => "http://resource.fis.plus/themes";
}

/// 获取主题加载器
///
/// [name]主题名称
FThemeLoader getThemeLoader(String name) {
  switch (FPlatform.current) {
    case FPlatformEnum.android:
    case FPlatformEnum.iOS:
      return FThemeLoaderNative(name);
    case FPlatformEnum.web:
      return FThemeLoaderWeb(name);
    case FPlatformEnum.webOnWin:
    case FPlatformEnum.webOnMac:
      return FThemeLoaderShellWeb(name);
  }
}
